#!/usr/bin/env python
#
#   __init__.py
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Run all tests."""

import unittest

import resmantest
import timertest
import vectortest

__author__ = 'Ross Light'
__date__ = 'July 26, 2006'
__all__ = ['resmantest',
           'timertest',
           'vectortest',
           'test_suite',]

test_suite = unittest.TestSuite([resmantest.test_suite,
                                 timertest.test_suite,
                                 vectortest.test_suite,])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#!/usr/bin/env python
#
#   ui.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Game user interface"""

import weakref

import pygame
from pygame.locals import *

from pymage import sprites

__author__ = 'Ross Light'
__date__ = 'July 20, 2006'
__all__ = ['left_align',
           'center_align',
           'right_align',
           'Widget',
           'Container',
           'TextWidget',
           'ImageWidget',
           'CursorWidget',
           'ButtonState',
           'StretchState',
           'Button',
           'PushButton',]
__docformat__ = 'reStructuredText'

# Constants

left_align = 0
center_align = 1
right_align = 2

class Widget(object):
    """Base class for all UI components."""
    def __init__(self,
                 parent=None,
                 rect=Rect(0, 0, 0, 0),
                 cache=False):
        self.__oldRect = None
        self.rect = rect
        self.__children = set()
        self.parent = parent
        self.__updates = [Rect(0, 0, self.rect.width, self.rect.height)]
        if cache:
            self.createCache()
        else:
            self.destroyCache()
        self.__active = False
    
    # Caching
    
    def createCache(self):
        """(Re)Creates a cache for the widget."""
        self.__cache = pygame.Surface(self.rect.size).convert_alpha()
        self.refresh()
    
    def destroyCache(self):
        """Destroys the widget's cache."""
        self.__cache = None
        self.refresh()
    
    def hasCache(self):
        """Returns whether the widget has a cache."""
        return bool(self.__cache is not None)
    
    # Child management
    
    def __len__(self):
        return len(self.__children)
    
    def __iter__(self):
        return iter(self.__children)
    
    def children(self):
        """Returns a list of all children."""
        return list(self.__children)
    
    def childTree(self, include_self=False, topdown=True):
        """
        Returns an iterator descending the entire child tree.
        
        ``includeSelf`` is a flag specifying whether the widget is included in
        the iterator.
        
        ``topdown`` is a flag specifying whether to go from widget to bottom
        or bottom to widget.
        """
        if include_self and topdown:
            yield self
        for child in self.__children:
            tree = child.childTree(include_self=True,
                                   topdown=topdown)
            for subchild in tree:
                yield subchild
        if include_self and not topdown:
            yield self
    
    def addChild(self, child):
        """Adds a child widget."""
        self.__children.add(child)
        child.parent = self
    
    def addChildren(self, children):
        """Adds children widgets."""
        self.__children.update(frozenset(children))
        for child in children:
            child.parent = self
    
    def removeChild(self, child):
        """Removes a child widget."""
        self.__children.remove(child)
        del child.parent
    
    def removeAllChildren(self):
        """Removes all child widgets."""
        children = list(self.__children)
        self.__children.clear()
        for child in children:
            del child.parent
    
    def _getParent(self):
        """Retrieves the widget's parent."""
        if self.__parent is None:
            return None
        else:
            parent = self.__parent()
            if parent is None:
                self.__parent = None
            return parent
    
    def _setParent(self, new_parent):
        """Changes the widget's parent, and removes it from the old one."""
        # Remove self from old parent
        self._delParent()
        # Add self to new parent
        if new_parent is None:
            self.__parent = new_parent
        else:
            if self not in new_parent.children():
                new_parent.addChild(self)
            self.__parent = weakref.ref(new_parent)
    
    def _delParent(self):
        """Remove the widget from its parent."""
        try:
            if self.__parent is not None:
                parent = self.__parent()
                if parent is not None and self in parent:
                    parent.removeChild(self)
        except AttributeError:
            pass
        self.__parent = None
    
    @property
    def root(self):
        """The root of the hierarchy."""
        widget = self
        while True:
            parent = widget.parent
            if parent is None:
                break
            widget = parent
        return widget
    
    parent = property(_getParent, _setParent, _delParent)
    
    # Event management
    
    def handle(self, event):
        """
        Handles a single event.
        
        Override to perform custom event-handling.  Return ``True`` if the event
        was handled successfully.
        """
        return False
    
    def processEvent(self, event):
        """
        Processes an event.
        
        The return value is whether the event was processed successfully.
        
        The event will start at a certain widget and traverse up the child
        hierarchy until the event is handled or the root is reached.  The
        starting widget is dependent on what the event type is:
        
        * If the event is a mouse event, the widget that the mouse is/was over.
        * Otherwise, the active widget.
        
        Do not override this method to perform event handling; instead, override
        the `handle` method.
        """
        if event.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP, MOUSEMOTION):
            tree = self.root.childTree(includeSelf=True,
                                       topdown=False)
            for child in tree:
                if child.screenRect().collidepoint(*event.pos):
                    widget = child
                    break
            else:
                widget = None
        else:
            widget = self.root.activeWidget()
        while widget is not None:
            if widget.handle(event):
                return True
            widget = widget.parent
        else:
            return False
    
    def canFocus(self):
        """
        Returns whether the widget can be focused.
        
        Default returns ``False``.
        """
        return False
    
    def focus(self, force_blur=False, force_focus=False):
        """
        Makes the widget the active one.
        
        Returns whether the focus was successful.
        
        You may extend this implementation to perform custom handling, but
        always call this implementation to actually focus the widget.
        """
        if force_focus or self.canFocus():
            activeWidget = self.root.activeWidget()
            if activeWidget is None or activeWidget.blur(force=force_blur):
                self.__active = True
                return True
            else:
                return False
        else:
            return False
    
    def canBlur(self):
        """
        Returns whether the widget can be blurred.
        
        Default returns ``True``.
        """
        return True
    
    def blur(self, force=False):
        """
        Makes the widget inactive.
        
        Returns whether the blur was successful.
        
        You may extend this implementation to perform custom handling, but
        always call this implementation to actually blur the widget.
        """
        if force or self.canBlur():
            self.__active = False
            return True
        else:
            return False
    
    def activeWidget(self):
        """
        Returns the current active widget, or ``None`` if there is not one.
        
        Be mindful that this only traverses from the receiver down.
        """
        for child in self.childTree(includeSelf=True):
            if child.isActive():
                return child
        else:
            return None
    
    def isActive(self):
        """Returns whether the widget is active."""
        return self.__active
    
    # Displaying
    
    def display(self, surface=None, origin=(0, 0)):
        """
        Display the widget on the given surface.
        
        The return value is a list of Rects that need to be updated.
        
        If no ``surface`` is given, the current display surface is used.
        
        If no ``origin`` is given, (0, 0) is used.  The origin should be in
        local space.
        
        You *shouldn't* have to override this method.  Instead, you should
        override the draw method.
        """
        screenUpdates = []
        if surface is None:
            surface = pygame.display.get_surface()
        # Add old bounds to screen updates
        if self.__oldRect != self.rect:
            if self.__oldRect is not None:
                screenUpdates.append(self.__oldRect.move(origin))
            self.rectChanged(self.__oldRect, Rect(self.rect))
            if self.hasCache():
                self.createCache()
            screenUpdates.append(self.rect.move(origin))
            self.__oldRect = Rect(self.rect)
        # Calculate dirty region
        if self.__updates:
            region = self.__updates[0].unionall(self.__updates[1:])
            del self.__updates[:]
        else:
            region = None
        # Draw self
        if self.hasCache():
            # Using cache
            drawSurf = self.__cache
            if region is not None:
                self.draw(drawSurf, region)
        else:
            # Not using cache
            drawSurf = pygame.Surface(self.rect.size).convert_alpha(surface)
            self.draw(drawSurf, self.bounds())
        surface.blit(drawSurf, self.rect.move(origin))
        # Determine screen updates
        if region is not None:
            screenUpdates.append(self.btlrect(region, origin))
        # Display children
        childOrigin = self.rect.move(origin).topleft
        for child in self.__children:
            screenUpdates.extend(child.display(surface, childOrigin))
        return screenUpdates
    
    def draw(self, surface, rect):
        """
        Performs the actual drawing.
        
        Default implementation clears the surface to a transparent color.
        """
        surface.fill((0, 0, 0, 0), rect)
    
    def refresh(self, rect=None):
        """
        Inform the widget that a given portion needs to be refreshed.
        
        If no rect is given, the entire widget will be refreshed.
        """
        if rect is None:
            rect = Rect(0, 0, self.rect.width, self.rect.height)
        self.__updates.append(Rect(rect))
    
    # Updating
    
    def update(self):
        """
        Updates the widget.
        
        Default implementation calls `update` on child widgets.
        """
        for child in self.__children:
            child.update()
    
    # Coordinates
    
    def stlrect(self, rect, origin=(0, 0)):
        """Converts from a screen rectangle to a local rectangle."""
        rect = Rect(rect)
        origin = self._localOrigin(origin)
        return rect.move(-origin[0], -origin[1])
    
    def stbrect(self, rect, origin=(0, 0)):
        """Converts from a screen rectangle to a bounds rectangle."""
        rect = Rect(rect)
        origin = self._boundsOrigin(origin)
        return rect.move(-origin[0], -origin[1])
    
    def ltsrect(self, rect, origin=(0, 0)):
        """Converts from a local rectangle to a screen rectangle."""
        rect = Rect(rect)
        origin = self._localOrigin(origin)
        return rect.move(origin[0], origin[1])
    
    def ltbrect(self, rect, origin=(0, 0)):
        """Converts from a local rectangle to a bounds rectangle."""
        rect = Rect(rect)
        origin = self.rect.topleft
        return rect.move(-origin[0], -origin[1])
    
    def btsrect(self, rect, origin=(0, 0)):
        """Converts from a bounds rectangle to a screen rectangle."""
        rect = Rect(rect)
        origin = self._boundsOrigin(origin)
        return rect.move(origin[0], origin[1])
    
    def btlrect(self, rect, origin=(0, 0)):
        """Converts from a bounds rectangle to a screen rectangle."""
        rect = Rect(rect)
        origin = self.rect.topleft
        return rect.move(origin[0], origin[1])
    
    def stlpoint(self, point, origin=(0, 0)):
        """Converts from a screen point to a local point."""
        origin = self._localOrigin(origin)
        return (point[0] - origin[0], point[1] - origin[1])
    
    def stbpoint(self, point, origin=(0, 0)):
        """Converts from a screen point to a bounds point."""
        origin = self._boundsOrigin(origin)
        return (point[0] - origin[0], point[1] - origin[1])
    
    def ltspoint(self, point, origin=(0, 0)):
        """Converts from a local point to a screen point."""
        origin = self._localOrigin(origin)
        return (point[0] + origin[0], point[1] + origin[1])
    
    def ltbpoint(self, point, origin=(0, 0)):
        """Converts from a local point to a bounds point."""
        origin = self.rect.topleft
        return (point[0] - origin[0], point[1] - origin[1])
    
    def btspoint(self, point, origin=(0, 0)):
        """Converts from a bounds point to a screen point."""
        origin = self._boundsOrigin(origin)
        return (point[0] + origin[0], point[1] + origin[1])
    
    def btlpoint(self, point, origin=(0, 0)):
        """Converts from a bounds point to a local point."""
        origin = self.rect.topleft
        return (point[0] + origin[0], point[1] + origin[1])
    
    def _localOrigin(self, origin=(0, 0)):
        """Returns the local origin as screen coordinates."""
        origin = self.screenRect(origin).topleft
        return (origin[0] - self.rect.x, origin[1] - self.rect.y)
    
    def _boundsOrigin(self, origin=(0, 0)):
        """Returns the bounds origin as screen coordinates."""
        return self.screenRect(origin).topleft
    
    def screenRect(self, origin=(0, 0)):
        """Calculates the screen rectangle."""
        rect = Rect(self.rect)
        parent = self.parent
        while parent is not None:
            rect.move_ip(parent.rect.topleft)
            parent = parent.parent
        rect.move_ip(origin)
        return rect
    
    def bounds(self):
        """Returns the rectangle that can be drawn in."""
        return Rect(0, 0, self.rect.width, self.rect.height)
    
    # Dynamic sizing
    
    def optimalSize(self):
        """
        Returns the optimal size of the widget.
        
        Default returns ``None``, but subclasses can override to return a value.
        """
        return None
    
    def pack(self, anchor=None):
        """
        Changes the widget's rect to the optimal size.
        
        If `optimalSize` returns None, this does nothing.
        
        ``anchor`` is a string specifying a corner/edge attribute name that will
        remain constant after the resize.  For best results, specify a corner
        (e.g. topleft).
        """
        if anchor is not None:
            anchorValue = getattr(self.rect, anchor)
        size = self.optimalSize()
        if size is not None:
            self.rect.size = size
            if anchor is not None:
                setattr(self.rect, anchor, anchorValue)
    
    def rectChanged(self, old_rect, new_rect):
        """
        Hook method for the rectangle changing.
        
        This is called before `display` does any drawing.
        """
        pass
    
    def _getRect(self):
        return self.__rect
    
    def _setRect(self, new_rect):
        self.__rect = Rect(new_rect)
    
    rect = property(_getRect, _setRect)

class Container(Widget):
    """A widget that is designed to contain other widgets."""
    def __init__(self,
                 border=0,
                 border_color=(0, 0, 0, 255),
                 bg_color=(0, 0, 0, 0),
                 *args, **kw):
        super(Container, self).__init__(*args, **kw)
        self.border = border
        self.borderColor = border_color
        self.bgColor = bg_color
    
    def draw(self, surface, rect):
        surface.fill(self.bgColor, rect)
        if self.border > 0:
            pygame.draw.rect(surface,
                             self.borderColor,
                             self.bounds(),
                             self.border)
    
    def optimalSize(self):
        childRects = [child.rect for child in self.children()]
        if childRects:
            allChildRect = childRects[0].union_all(childRects[1:])
            return (allChildRect.x + allChildRect.width,
                    allChildRect.y + allChildRect.height)
        else:
            return None

class TextWidget(Widget):
    """A widget that can display a single line of text."""
    fgColor = (0, 0, 0, 255)
    bgColor = (0, 0, 0, 0)
    antialias = True
    shadow = False
    shadowColor = (0, 0, 0, 128)
    
    def __init__(self, text='', font=None, *args, **kw):
        """
        Initializes a text widget.
        
        Three additional parameters can be specified via keyword arguments:
        
        - ``fgColor``: The color of the rendered text.  Default is black.
        - ``bgColor``: The background color of the text.  Default is clear.
        - ``antialias``: Whether to antialias.  Default is ``True``.
        - ``align``: How to align the text.  Default is `left_align`.
        - ``shadow``: Whether the text has a drop shadow.  Default is ``False``.
        - ``shadowColor``: The drop shadow's color.  Default is a
          half-transparent black.
        """
        # Get keyword options
        self.fgColor = kw.pop('fgColor', self.fgColor)
        self.bgColor = kw.pop('bgColor', self.bgColor)
        self.antialias = kw.pop('antialias', self.antialias)
        self.align = kw.pop('align', left_align)
        self.shadow = kw.pop('shadow', self.shadow)
        # Perform superclass initialization
        super(TextWidget, self).__init__(*args, **kw)
        # Perform initialization
        self.text = text
        if font is None:
            font = pygame.font.Font(None, 14)
        self.font = font
    
    def draw(self, surface, rect):
        # Render text
        surface.fill(self.bgColor, rect)
        rendered = self.font.render(self.text, self.antialias, self.fgColor)
        # Determine render position
        textRect = rendered.get_rect()
        alignPoints = {left_align: 'topleft',
                       center_align: 'midtop',
                       right_align: 'topright',}
        setattr(textRect, alignPoints[self.align],
                getattr(self.bounds(), alignPoints[self.align]))
        # Render shadow
        if self.shadow:
            renderedShadow = self.font.render(self.text,
                                              self.antialias,
                                              self.shadowColor)
            shadowRect = textRect.move(0, 2)
        else:
            renderedShadow = None
            shadowRect = None
        # Blit text to screen, if necessary
        if renderedShadow is not None and shadowRect.colliderect(rect):
            surface.blit(renderedShadow, shadowRect)
        if textRect.colliderect(rect):
            surface.blit(rendered, textRect)
    
    def optimalSize(self):
        return self.font.size(self.text)
    
    def _getText(self):
        """Retrieve text value."""
        return self._text
    
    def _setText(self, text):
        """Change text value and refresh."""
        self._text = text
        self.refresh()
    
    def _getFont(self):
        """Retrieve font."""
        return self._font
    
    def _setFont(self, font):
        """Change font and refresh."""
        self._font = font
        self.refresh()
    
    def _getAlign(self):
        """Retrieve alignment."""
        return self._align
    
    def _setAlign(self, align):
        """Change alignment and validate."""
        if align in (left_align, center_align, right_align):
            self._align = align
        else:
            raise ValueError("Alignment %r is not valid" % align)
    
    text = property(_getText, _setText)
    font = property(_getFont, _setFont)
    align = property(_getAlign, _setAlign)

class ImageWidget(Widget):
    """A widget that displays an image."""
    def __init__(self, image, stretch=False, *args, **kw):
        super(ImageWidget, self).__init__(*args, **kw)
        self.image = image
        self.stretch = stretch
    
    def draw(self, surface, rect):
        super(ImageWidget, self).draw(surface, rect)
        image = self.image
        if self.stretch and self.rect.size != image.get_size():
            image = pygame.transform.scale(self.image, rect.size)
        surface.blit(image, (0, 0), rect)
    
    def getImage(self):
        """
        Retrieves the current image.
        
        You can also use the ``image`` property.
        """
        return self.__image
    
    def setImage(self, image, tryIM=True):
        """
        Changes the current image.
        
        You can also use the ``image`` property, but it always tries
        `sprites.ImageManager`.
        """
        self.__image = sprites.getImage(image, tryIM)
        self.refresh()
    
    def optimalSize(self):
        return self.image.get_size()
    
    image = property(getImage, setImage)

class CursorWidget(ImageWidget):
    """A widget that displays an image, following the cursor."""
    def __init__(self, hotspot=(0, 0), *args, **kw):
        super(CursorWidget, self).__init__(*args, **kw)
        self.hotspot = hotspot
        self.followMouse()
    
    def handle(self, event):
        if event.type == MOUSEMOTION:
            self.followMouse(event.pos)
            return True
        else:
            return False
    
    def followMouse(self, pos=None):
        """Called to follow the mouse."""
        if pos is None:
            pos = pygame.mouse.get_pos()
        self.rect.topleft = self.stlpoint((pos[0] - self.hotspot[0],
                                           pos[1] - self.hotspot[1]))

class ButtonState(object):
    """A button state."""
    def setState(self, button):
        """Called to change the button into the given state."""
        pass
    
    def unsetState(self, button):
        """Called to undo the effects of `setState`."""
        pass
    
    def optimalSize(self):
        """Returns the optimal size of the button, or ``None``."""
        return None

class StretchState(ButtonState):
    """
    A button state in which three images are used.
    
    Two are used for caps, and the center is stretched to fit the text.
    """
    def __init__(self, text, textColor, imgL, imgC, imgR):
        self.text = text
        self.textColor = textColor
        self.imgL = sprites.getImage(imgL)
        self.imgC = sprites.getImage(imgC)
        self.imgR = sprites.getImage(imgR)
    
    def setState(self, button):
        # Calculate left cap rect
        lrect = Rect(0, 0, 0, 0)
        lrect.size = self.imgL.get_size()
        lrect.topleft = (0, 0)
        # Calculate right cap rect
        rrect = Rect(0, 0, 0, 0)
        rrect.size = self.imgR.get_size()
        rrect.topright = (button.rect.width, 0)
        # Calculate center cap rect
        crect = Rect(0, 0, 0, 0)
        crect.height = self.imgC.get_height()
        crect.width = rrect.left - lrect.right
        crect.topleft = lrect.topright
        # Add children
        ImageWidget(self.imgL,
                    parent=button,
                    rect=lrect)
        ImageWidget(self.imgC,
                    stretch=True,
                    parent=button,
                    rect=crect)
        ImageWidget(self.imgR,
                    parent=button,
                    rect=rrect)
        TextWidget(self.text,
                   font=self.getFont(),
                   align=center_align,
                   fgColor=self.textColor,
                   parent=button,
                   rect=crect)
    
    def unsetState(self, button):
        button.removeAllChildren()
        
    def getFont(self):
        return pygame.font.Font(None, self.imgC.get_height())
    
    def optimalSize(self):
        font = self.getFont()
        width = (self.imgL.get_width() +
                 font.size(self.text)[0] +
                 self.imgR.get_width())
        height = self.imgC.get_height()
        return (width, height)

class Button(Widget):
    """Abstract superclass for buttons."""
    def __init__(self,
                 state,
                 callback=None,
                 args=[],
                 kw={},
                 *initArgs, **initKw):
        self.nav_up = initKw.pop('up', None)
        self.nav_down = initKw.pop('down', None)
        self.nav_left = initKw.pop('left', None)
        self.nav_right = initKw.pop('right', None)
        super(Button, self).__init__(*initArgs, **initKw)
        self.setState(state)
        self.callback = callback
        self.callArgs = args
        self.callKw = kw
    
    def handle(self, event):
        keyButtons = {K_UP: self.nav_up,
                      K_DOWN: self.nav_down,
                      K_LEFT: self.nav_left,
                      K_RIGHT: self.nav_right,}
        hatButtons = {(0, 1): self.nav_up,
                      (0, -1): self.nav_down,
                      (-1, 0): self.nav_left,
                      (1, 0): self.nav_right,}
        if event.type == KEYUP and keyButtons.get(event.key) is not None:
            keyButtons[event.key].focus()
            return True
        elif event.type == JOYHATMOTION and \
             hatButtons.get(event.value) is not None:
            hatButtons[event.value].focus()
            return True
        else:
            return False
    
    def setState(self, new_state):
        try:
            oldState = self.state
        except AttributeError:
            pass
        else:
            if oldState == new_state:
                return
            oldState.unsetState(self)
        self.state = new_state
        self.state.setState(self)
    
    def perform(self):
        """Calls the button's callback."""
        if self.callback is not None:
            self.callback(self, *self.callArgs, **self.callKw)
    
    def optimalSize(self):
        """Sizes the button to the optimum size."""
        return self.state.optimalSize()
    
    def rectChanged(self, old_rect, new_rect):
        self.state.unsetState(self)
        self.state.setState(self)
    
    def __getattribute__(self, name):
        value = super(Button, self).__getattribute__(name)
        if name.startswith('nav_') and value is not None:
            value = value()
            if value is None:
                setattr(self, name, None)
        return value
    
    def __setattr__(self, name, value):
        if name.startswith('nav_') and value is not None:
            value = weakref.ref(value)
        super(Button, self).__setattr__(name, value)

class PushButton(Button):
    def __init__(self,
                 normState,
                 rollState=None,
                 *args, **kw):
        super(PushButton, self).__init__(normState, *args, **kw)
        self.normState = normState
        if rollState is None:
            self.rollState = self.normState
        else:
            self.rollState = rollState
        self._lastMousePos = pygame.mouse.get_pos()
    
    def handle(self, event):
        if super(PushButton, self).handle(event):
            return True
        elif (event.type == MOUSEBUTTONUP and event.button == 1) or \
             (event.type == KEYUP and event.key == K_RETURN) or \
             (event.type == JOYBUTTONUP and event.button == 0):
            self.perform()
            return True
        else:
            return False
    
    def update(self):
        mousePos = pygame.mouse.get_pos()
        if mousePos != self._lastMousePos:
            self._lastMousePos = mousePos
            mouseInButton = self.screenRect().collidepoint(mousePos)
            if not self.isActive() and mouseInButton:
                self.focus()
            elif self.isActive() and not mouseInButton:
                self.blur()
    
    def canFocus(self):
        return True
    
    def focus(self, *args, **kw):
        success = super(PushButton, self).focus(*args, **kw)
        if success:
            self.setState(self.rollState)
        return success
    
    def blur(self, *args, **kw):
        success = super(PushButton, self).blur(*args, **kw)
        if success:
            self.setState(self.normState)
        return success



#!/usr/bin/env python
#
#   sprites.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Defines game sprites"""

import warnings

import pygame
from pygame.locals import *

from pymage import resman

__author__ = 'Ross Light'
__date__ = 'May 22, 2006'
__all__ = ['getImage',
           'ImageManager',
           'im',
           'Sprite',
           'Animation',]
__docformat__ = 'reStructuredText'

def getImage(image, tryIM=True):
    """
    Retrieves an image.
    
    Provides a standard way to either pass an image or a string to a function.
    """
    if isinstance(image, pygame.Surface):
        return image
    else:
        if tryIM:
            try:
                return im.load(image)
            except ValueError:
                pass
        return pygame.image.load(image)

class ImageManager(resman.Submanager):
    """Game image manager."""
    resourceType = resman.ImageResource

    def loadImage(self, *args, **kw):
        """
        Loads an image from disk, using a cached representation, if possible.
        
        .. Warning::
           `loadImage` method is deprecated, for favor of the new Submanager
           API.  Use `load` instead.
        """
        warnings.warn("loadImage is deprecated; use load.",
                      DeprecationWarning,
                      stacklevel=2)
        return self.load(*args, **kw)

im = ImageManager()

class Sprite(pygame.sprite.Sprite, object):
    """Abstract superclass for sprites."""
    hpadding = vpadding = 0
    angleTolerance = 0.5
    clamp = True
    
    def __init__(self, image=None):
        """
        Initializes a sprite.
        
        If the ``image`` parameter is omitted, the class variable ``image`` will
        be used.  If the parameter it finally receives is a string, it will try
        to load the image from the `ImageManager`.  If it is not in the
        `ImageManager`, it assumes that it is a path or pygame.Surface.
        """
        pygame.sprite.Sprite.__init__(self)
        if image is None:
            image = self.image
        self.setImage(image)
        self.rect = self.image.get_rect()
        self.area = pygame.display.get_surface().get_rect()
        self.angle = 0.0
    
    def setImage(self, image, tryIM=True):
        """Changes the current image and the revert image variables."""
        self.image = self._image = getImage(image, tryIM).convert_alpha()
    
    def collideBox(self):
        """
        Returns the box used for checking with the `touches` method.
        
        By default, this uses the ``hpadding`` and ``vpadding`` to construct an
        inset box.  Override to have a different collide box.
        """
        # Multiply by two to get all-around coverage
        return self.rect.inflate(self.hpadding * -2, self.vpadding * -2)
    
    def touches(self, other):
        """
        Returns whether the other sprite is actually touching the receiver.
        """
        return self.collideBox().colliderect(other.collideBox())
    
    def updateWithVector(self, vector, clamp=None):
        """Moves the sprite with the given vector."""
        self.rect.x += vector.x
        self.rect.y += vector.y
        if clamp is None:
            clamp = self.clamp
        if clamp:
            self.rect = self.rect.clamp(self.area)
    
    def rotateImage(self):
        """Rotates the sprite's image to the proper angle."""
        if abs(self.angle) < self.angleTolerance:
            self.image = self._image
        else:
            self.image = pygame.transform.rotate(self._image, self.angle)
        self.rect.size = self.image.get_size()

class Animation(Sprite):
    """Superclass for ambient animations."""
    loop = False
    
    def __init__(self, frames=None, loop=None):
        """
        Initializes an animation.
        
        - ``frames`` is a list of pygame.Surfaces or strings.  If not specified,
          it uses the ``frames`` class variable.
        - ``loop`` is a flag specifying whether the animation should
          continuously play or whether it should kill itself after one play.  If
          not specified, it uses the ``loop`` class variable.
        """
        if frames is None:
            frames = self.frames
        self.frames = list(frames)
        if loop is not None:
            self.loop = loop
        self.frameNum = 0
        super(Animation, self).__init__(frames[0])
    
    def update(self):
        """
        Updates the sprite.
        
        Default implementation advances to the next frame.
        """
        self.advance()
    
    def advance(self):
        """Advances to the next frame and dies if it reaches the end."""
        self.frameNum += 1
        if self.frameNum >= len(self.frames):
            if self.loop:
                self.frameNum = 0
            else:
                self.kill()
        else:
            self.setImage(self.frames[self.frameNum])



#!/usr/bin/env python
#
#   resman.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Handles all resources"""

import warnings

import pygame

__author__ = 'Ross Light'
__date__ = 'February 19, 2007'
__all__ = ['ResourceManager',
           'resman',
           'Resource',
           'ImageResource',
           'AudioResource',
           'SoundResource',
           'MusicResource',
           'Submanager',]
__docformat__ = 'reStructuredText'

class ResourceManager(object):
    """
    Master resource manager.
    
    The resource manager does not directly load resources, but it stores
    resource information for later use.  All of the resources should be added
    to the manager (via `addResource`) when the program starts.  Each resource
    will be assigned a key, which is what is used to access that resource from
    then on.  The resource can then be loaded, cached, and accessed by that key.
    
    One can also create cache groups, sets of resources that can be cached and
    uncached in bulk.  Multiple cache groups can have the same resource in them;
    when no cache groups are caching a given resource, the resource's cache is
    destroyed.
    """
    def __init__(self):
        self.resources = {}
        self.cacheGroups = {}
        self.cacheCount = {}
    
    def cleanup(self):
        """Manually destroy all resources."""
        for key in self.cacheCount:
            self.getResource(key).destroyCache()
        self.resources = {}
        self.cacheGroups = {}
        self.cacheCount = {}
    
    # Resource primitives #
    
    def addResource(self, key, resource):
        """
        Adds a resource to the manager.
        
        A ``KeyError`` is raised if a resource of the same key already exists.
        """
        if key not in self.resources:
            self.resources[key] = resource
        else:
            raise KeyError(key)
    
    def getResource(self, key):
        """
        Retrieves a resource from the manager.
        
        A ``KeyError`` is raised if no resource exists with that key.
        """
        if self.hasResource(key):
            return self.resources[key]
        else:
            raise KeyError(key)
    
    def hasResource(self, key):
        """Query the existence of the resource."""
        return key in self.resources
    
    def removeResource(self, key):
        """
        Removes a resource from the manager.
        
        A ``KeyError`` is raised if no resource exists with that key.
        """
        if key in self.resources:
            del self.resources[key]
        else:
            raise KeyError(key)
    
    # Resource operations #
    
    def cacheResource(self, key, force=False):
        """
        Caches the resource.
        
        This increases the cache count of the resource, so it will not be
        uncached by resource groups.
        
        The ``force`` flag has the exact same meaning as the one in
        `Resource.createCache`.
        """
        self.getResource(key).createCache(force=force)
        try:
            self.cacheCount[key] += 1
        except KeyError:
            self.cacheCount[key] = 1
    
    def uncacheResource(self, key):
        """
        Destroys the resource's cache.
        
        This decreases the cache count of the resource, so it will not be
        uncached until all resource groups are uncached.
        """
        try:
            self.cacheCount[key] -= 1
        except KeyError:
            self.cacheCount[key] = 0
        if self.cacheCount[key] <= 0:
            self.getResource(key).destroyCache()
    
    def loadResource(self, key, *args, **kw):
        """
        Loads the resource with the given key.
        
        Any additional arguments are passed to the resource's `Resource.get`
        method.
        """
        return self.getResource(key).get(*args, **kw)
    
    # Cache group primitives #
    
    def addCacheGroup(self, key, resource_keys):
        """
        Adds a cache group to the manager.
        
        A ``KeyError`` is raised if a cache group of the same key already
        exists.
        """
        if key not in self.cacheGroups:
            self.cacheGroups[key] = frozenset(resource_keys)
        else:
            raise KeyError(key)
    
    def getCacheGroup(self, key):
        """
        Retrieves a cache group from the manager.
        
        A ``KeyError`` is raised if no cache group exists with that key.
        """
        if self.hasCacheGroup(key):
            return self.cacheGroups[key]
        else:
            raise KeyError(key)
    
    def hasCacheGroup(self, key):
        """Query the existence of the cache group."""
        return key in self.cacheGroups
    
    def removeCacheGroup(self, key):
        """
        Removes a cache group from the manager.
        
        A ``KeyError`` is raised if no cache group exists with that key.
        """
        if key in self.cacheGroups:
            del self.cacheGroups[key]
        else:
            raise KeyError(key)
    
    # Cache group operations #
    
    def cacheGroup(self, key, force=False):
        """
        Caches the resources in a cache group.
        
        The ``force`` flag has the exact same meaning as the one in
        `Resource.createCache`.
        """
        for cacheKey in self.getCacheGroup(key):
            self.cacheResource(cacheKey, force=force)
    
    def uncacheGroup(self, key):
        """
        Uncaches the resources in a cache group.
        
        Because multiple groups may reference the same resource, you should not
        depend on all of the resources being uncached.
        """
        for cacheKey in self.getCacheGroup(key):
            self.uncacheResource(cacheKey)

resman = ResourceManager()

class Resource(object):
    """Generic resource object."""
    def __init__(self, path):
        self.path = path
        self.cache = None
    
    def load(self):
        """
        Load the resource.
        
        The default implementation is not implemented, so it must be overridden
        in subclasses.  The return value should be something immediately usable,
        such as a ``pygame.Surface`` object.
        
        This is usually not called directly, instead, see the `get` method.
        """
        raise NotImplementedError()
    
    def get(self, *args, **kw):
        """
        Get the resource's content.
        
        The cache is returned, if it exists.  Otherwise, the resource is loaded,
        but no cache is saved.
        
        Any additional arguments are passed along to the `load` method.
        """
        if self.hasCache():
            return self.cache
        else:
            return self.load(*args, **kw)
    
    def createCache(self, *args, **kw):
        """
        Creates a resource cache.
        
        The default implementation passes arguments to the `load` method and
        stores the result to the ``cache`` attribute.
        
        The ``force`` keyword will specify whether to refresh the cache if the
        cache already exists.
        """
        force = kw.pop('force', False)
        if force or self.cache is None:
            self.cache = self.load(*args, **kw)
    
    def hasCache(self):
        """Returns ``True`` if the resource has a cache."""
        return self.cache is not None
    
    def destroyCache(self):
        """
        Destroys the resource cache.
        
        The default implementation sets the ``cache`` attribute to ``None``.
        """
        self.cache = None

class ImageResource(Resource):
    """Image resource loader."""
    def __init__(self, path, convert=True, alpha=True):
        """
        Initializes the resource.
        
        In addition to ``path``, there are two parameters, ``convert`` and
        ``alpha``. ``convert`` specifies whether the resource should be
        converted to the screen format after being loaded. ``alpha`` specifies
        whether alpha information should be preserved, but the flag is ignored
        if ``convert`` is ``False``.
        """
        super(ImageResource, self).__init__(path)
        self.convert = convert
        self.alpha = alpha
    
    def load(self):
        """Load the image."""
        img = pygame.image.load(self.path)
        if self.convert:
            if self.alpha:
                return img.convert_alpha()
            else:
                return img.convert()
        else:
            return img

class AudioResource(Resource):
    """Encapsulates all audio resources."""
    pass

class SoundResource(AudioResource):
    """Sound resource loader."""
    def load(self):
        """Load the sound."""
        return pygame.mixer.Sound(self.path)

class MusicResource(AudioResource):
    """
    Music resource loader.
    
    The `load` method returns ``None``, as the loader loads the music into
    pygame's music mixer.
    """
    def load(self):
        """Load the music."""
        pygame.mixer.music.load(self.path)
    
    def createCache(self, *args, **kw):
        """Music doesn't support caching."""
        raise TypeError("Music doesn't support caching")
    
    def destroyCache(self):
        """Music doesn't support caching."""
        raise TypeError("Music doesn't support caching")

class Submanager(object):
    """
    Slave resource manager.
    
    Submanagers load resources from a central resource manager, but only deal
    with certain types.  They cannot modify the resources, only control their
    loading and caching.
    """
    resourceType = Resource
    
    def __init__(self, manager=None):
        """
        Initializes the submanager.
        
        Default uses the master resource manager
        (i.e. ``pymage.resman.resman``).
        """
        if manager is None:
            manager = resman
        self.manager = manager
    
    def prepare(self, tag, defaultPath=None):
        """
        Declares a tag for later use.
        
        .. Warning::
           `prepare` is retained for compatibility reasons.  You should be using
           `ResourceManager.addResource` to add new resources.
        """
        warnings.warn("prepare is deprecated; use ResourceManager.addResource.",
                      DeprecationWarning,
                      stacklevel=2)
        if defaultPath is None:
            defaultPath = tag
        resource = self.resourceType(defaultPath)
        self.manager.addResource(tag, resource)
    
    def cache(self, key, force=False):
        """Cache the resource and return the resource's content."""
        resource = self.manager.getResource(key)
        if isinstance(resource, self.resourceType):
            self.manager.cacheResource(key, force)
            return self.manager.loadResource(key)
        else:
            raise KeyError(key)
    
    def uncache(self, key):
        """Uncache the resource."""
        resource = self.manager.getResource(key)
        if isinstance(resource, self.resourceType):
            self.manager.uncacheResource(key)
        else:
            raise KeyError(key)
    
    def load(self, key, *args, **kw):
        """Retreive's a resource's content."""
        resource = self.manager.getResource(key)
        if isinstance(resource, self.resourceType):
            return self.manager.loadResource(key, *args, **kw)
        else:
            raise KeyError(key)
    


#!/usr/bin/env python
#
#   joystick.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Manage joysticks"""

import pygame
from pygame.locals import *

__author__ = 'Ross Light'
__date__ = 'July 20, 2006'
__all__ = ['Axis']
__docformat__ = 'reStructuredText'

class Axis(object):
    """
    Represents a joystick axis.
    
    There is an attribute, ``perfect``, that is defaulted to ``False``.  When
    ``True``, a few extra calculations are performed to ensure that the axis is
    always within the -1.0 to 1.0 range.  Otherwise, the only calculation
    performed is inversion.
    """
    perfectRange = (-1.0, 0.0, 1.0)
    perfect = False
    
    def __init__(self, joy, num, invert=False, perfect=None):
        self.joy, self.num = joy, num
        self.invert = invert
        if perfect is not None:
            self.perfect = perfect
        if self.perfect:
            self.min, self.max = 0.0, 0.0
            self.offset, self.scale = 0.0, 1.0
    
    def addEntry(self, value):
        """
        If the axis is in perfect mode, calibrate the axis from the information
        given.
        
        You shouldn't need to use this, instead, use `sample`.
        """
        if self.perfect:
            perfectMin, perfectMid, perfectMax = self.perfectRange
            if value > self.max:
                self.max = value
            elif value < self.min:
                self.min = value
            else:
                return
            # Do cached calculations
            mid = (self.max + self.min) / 2
            self.offset = perfectMid - mid
            self.scale = (perfectMax - perfectMin) / (self.max - self.min)
    
    def sample(self):
        """
        Samples calibration information for the axis.
        
        This method only needs to be called in perfect mode.  It is not
        neccessary in normal mode, but shouldn't take a performance hit, so you
        should probably stick it in your code anyway.
        """
        joy = pygame.joystick.Joystick(self.joy)
        self.addEntry(joy.get_axis(self.num))
    
    def convert(self, raw_value):
        """
        Converts a raw value from the joystick.
        
        You shouldn't need to use this, instead, use `get`.
        """
        if self.perfect:
            value = (raw_value + self.offset) * self.scale
        else:
            value = raw_value
        if self.invert:
            value = -value
        return value
    
    def get(self):
        """
        Retrieves the current value of the axis.
        
        If the axis is in perfect mode, this performs calculations based on
        previous calibration information to ensure it is perfect.  If in normal
        mode, this retreives the raw value (inverted, if necessary).
        """
        joy = pygame.joystick.Joystick(self.joy)
        return self.convert(joy.get_axis(self.num))



#!/usr/bin/env python
#
#   resmantest.py
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

import unittest

from pymage.timer import *

__author__ = 'Ross Light'
__date__ = 'May 24, 2007'
__all__ = ['TimerTestCase', 'test_suite',]

class TimerTestCase(unittest.TestCase):
    duration = 5.0
    magic1, magic2 = 'xyzzy', 'ROSEbud'
    
    def setUp(self):
        pass
    
    def callback(self, arg1, arg2):
        self.called = True
        self.assert_(arg1 is self.magic1, "Positional argument lost")
        self.assert_(arg2 is self.magic2, "Keyword argument lost")
    
    def testBasicTimer(self):
        """Basic timer test"""
        timer = Timer(self.duration)
        fireCount = timer.update(self.duration / 2)
        self.assertEqual(fireCount, 0, "Timer fired too early!")
        fireCount = timer.update(self.duration / 2)
        self.assertEqual(fireCount, 1, "Timer firing problem")
        fireCount = timer.update(42)
        self.assertEqual(fireCount, 0, "Timer is trigger-happy")
    
    def testCallback(self):
        """Timer callback test"""
        self.called = False
        timer = Timer(self.duration, 0, self.callback,
                      (self.magic1,), {'arg2': self.magic2})
        fireCount = timer.update(self.duration)
        self.assert_(self.called, "Callback was never called!")
        self.assertEqual(fireCount, 1, "Timer firing problem")
    
    def testLoop(self):
        """Timer loop test"""
        timer = Timer(self.duration, 3)
        fireCount = timer.update(self.duration)
        self.assertEqual(fireCount, 1, "Timer didn't fire right first")
        fireCount = timer.update(self.duration)
        self.assertEqual(fireCount, 1, "Timer didn't fire right second")
        fireCount = timer.update(self.duration * 2)
        self.assertEqual(fireCount, 2, "Timer didn't fire twice")
        fireCount = timer.update(self.duration)
        self.assertEqual(fireCount, 0, "Timer is stuck")

test_suite = unittest.makeSuite(TimerTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#!/usr/bin/env python
#
#   __init__.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Various utilities to simplify pygame development."""

__author__ = 'Ross Light'
__date__ = 'July 20, 2006'
__all__ = ['config',
           'joystick',
           'resman',
           'sound',
           'sprites',
           'states',
           'timer',
           'ui',
           'vector',]
__docformat__ = 'reStructuredText'

from pymage import config
from pymage import joystick
from pymage import resman
from pymage import sound
from pymage import sprites
from pymage import states
from pymage import timer
from pymage import ui
from pymage import vector



#!/usr/bin/env python
#
#   timer.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Timing utilities"""

from __future__ import division

__author__ = 'Ross Light'
__date__ = 'January 5, 2007'
__all__ = ['Timer',
           'TimerGroup',]
__docformat__ = 'reStructuredText'

class Timer(object):
    """Timer that calls a callback after a given amout of time."""
    def __init__(self,
                 duration,
                 loop=0,
                 callback=None,
                 callArgs=None,
                 callKw=None):
        """
        Initializes the timer.

        ``duration`` is the length of time, in seconds, before the timer fires.

        ``loop`` specifies whether the timer should fire multiple times.  If the
        parameter is 0 (the default), the timer fires only once.  If the
        parameter is a positive integer, it fires that many times plus one.
        Otherwise, the timer fires indefinitely.

        ``callback`` is the function to be called when the timer fires.  It
        usually takes no arguments, but additional ones can be specified using
        ``callArgs`` and ``callKw``.
        """
        self.callback = callback
        if callArgs is None:
            self.callArgs = []
        else:
            self.callArgs = list(callArgs)
        if callKw is None:
            self.callKw = {}
        else:
            self.callKw = dict(callKw)
        self.duration = float(duration)
        self.loop = loop
        self.reset()

    def reset(self, duration=None):
        """
        Resets the timer.

        If ``duration`` is specified, the timer's duration is set to the given
        value.
        """
        if duration is not None:
            self.duration = float(duration)
        if self.duration < 0:
            raise ValueError("Duration must be greater than 0")
        self.value = self.duration

    def update(self, time):
        """
        Updates the timer count.

        Returns how many times the timer fired.

        The value of ``time`` can be a number, in which case it is assumed to be
        the elapsed time in seconds; an object with a ``get_time`` method (such
        as a ``pygame.time.Clock`` instance); or an object with a ``clock``
        attribute, which should provide a ``get_time`` method.  The ``get_time``
        should return the elapsed time in milliseconds.
        """
        # If we're done, do nothing
        if self.loop == 0 and self.value == 0:
            return 0
        # Get the time elapsed
        if hasattr(time, 'get_time'):
            time = time.get_time() / 1000
        elif hasattr(time, 'clock'):
            time = time.clock.get_time() / 1000
        else:
            try:
                time = float(time)
            except TypeError:
                raise TypeError("Unknown type given to Timer.update()")
        # Fire as many times as necessary
        fireCount = 0
        self.value -= time
        while self.value <= 0:
            fireCount += 1
            self.value += self.duration
            if self.callback is not None:
                self.callback(*self.callArgs, **self.callKw)
            if self.loop == 0:
                self.value = 0.0
                break
            elif self.loop > 0:
                self.loop -= 1
        return fireCount

class TimerGroup(object):
    """A group of `Timer` objects that all update with one call."""
    def __init__(self, timers=None):
        if timers is None:
            timers = []
        self.timers = list(timers)
    
    def add(self, timer):
        """Adds a timer to the group."""
        self.timers.append(timer)
    
    def create(self, *args, **kw):
        """
        Creates a new timer and adds it to the group.
        
        The arguments are the same as `Timer.__init__`.
        """
        self.add(Timer(*args, **kw))
    
    def remove(self, timer):
        """Removes a timer from the group."""
        self.timers.remove(timer)
    
    def update(self, time):
        """Updates all of the timers."""
        for timer in self:
            timer.update(time)
    
    def __len__(self):
        return len(self.timers)
    
    def __iter__(self):
        return iter(self.timers)



#!/usr/bin/env python
#
#   builddoc.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Build API documentation with epydoc"""

import os
import posixpath
import sys
import tarfile
import zipfile

__author__ = 'Ross Light'
__date__ = 'July 27, 2007'

progname = os.path.basename(sys.argv[0])
outputDirFormat = 'pymage-api-%s'
eggInfoName = 'pymage.egg-info'
pkgInfoName = 'PKG-INFO'
versionSymbol = 'Version:'
projectURL = 'http://code.google.com/p/pymage/'

class ProgramError(Exception):
    pass

def create_tgz(directory, output):
    t = tarfile.open(output, 'w:gz')
    t.add(directory, os.path.basename(directory))
    t.close()

def create_zip(directory, output):
    def pack(archive, paths, archivePath=None):
        for path in paths:
            if archivePath is None:
                archivePath = os.path.basename(path)
            if os.path.isdir(path):
                for entry in os.listdir(path):
                    pack(archive,
                         [os.path.join(path, entry)],
                         posixpath.join(archivePath, entry))
            else:
                archive.write(path, archivePath)
    z = zipfile.ZipFile(output, 'w', zipfile.ZIP_DEFLATED)
    pack(z, [directory])
    z.close()

try:
    # Ensure we have epydoc
    try:
        import epydoc
        from epydoc import log
        from epydoc.docbuilder import build_doc_index
        from epydoc.docwriter.html import HTMLWriter
    except ImportError:
        raise ProgramError("epydoc must be installed to build documentation")
    # Set up logging
    class BuildDocLogger(log.Logger):
        def __init__(self, threshold=log.WARNING):
            self.threshold = threshold
        
        def start_progress(self, header=None):
            if '-q' not in sys.argv[1:]:
                if header:
                    print "%s:" % header
                else:
                    print "epydoc task:"
        
        def progress(self, percent, message=''):
            if '-q' not in sys.argv[1:]:
                indent = ' ' * 2
                if message:
                    print indent + "epydoc: %s (%i%% completed)" % \
                                   (message, percent * 100)
                else:
                    print indent + "epydoc: %i%% completed" % (percent * 100)
        
        def end_progress(self):
            pass
        
        def log(self, level, message):
            if level >= self.threshold:
                print "%s: epydoc: %s" % (progname, message)
    logger = BuildDocLogger()
    log.register_logger(logger)
    # Make sure we're in the pymage source directory
    if not os.path.exists(os.path.join(os.getcwd(), eggInfoName)):
        raise ProgramError("Must be run in source directory")
    # Get version number
    pkgInfoFile = open(os.path.join(os.getcwd(), eggInfoName, pkgInfoName))
    try:
        for line in pkgInfoFile:
            if line.startswith(versionSymbol):
                version = line[len(versionSymbol):].strip()
                break
        else:
            raise ProgramError("Unable to find version number")
    finally:
        pkgInfoFile.close()
    # Create destination directory (if necessary)
    outputDir = os.path.join(os.getcwd(), outputDirFormat % version)
    if os.path.exists(outputDir):
        if not os.path.isdir(outputDir):
            raise ProgramError("Output path is not a directory")
    else:
        print "Creating '%s'..." % outputDir
        os.mkdir(outputDir)
    # Build documentation
    docIndex = build_doc_index(['pymage',])
    if docIndex is None:
        raise ProgramError("Documentation indexing failed")
    writer = HTMLWriter(docIndex,
                        prj_name='pymage',
                        prj_url=projectURL,
                        show_private=False,
                        show_imports=False,
                        include_source_code=False,)
    print "Writing documentation..."
    writer.write(outputDir)
    # Make archives
    print "Creating Tar-Gzip archive..."
    create_tgz(outputDir, (os.path.extsep).join([outputDir, 'tar', 'gz']))
    print "Creating Zip archive..."
    create_zip(outputDir, outputDir + os.path.extsep + 'zip')
except ProgramError, e:
    msg = str(e)
    msg = msg[0].lower() + msg[1:]
    print >> sys.stderr, "%s: %s" % (progname, str(e))
    sys.exit(1)
except Exception, e:
    print >> sys.stderr, "%s: internal error: %s" % (progname, str(e))
    sys.exit(-1)
else:
    sys.exit(0)



#!/usr/bin/env pythonw
#
#   sound.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""pymage sound subsystem"""

import warnings

import pygame
from pygame.locals import *

from pymage import resman

__author__ = 'Ross Light'
__date__ = 'July 19, 2006'
__all__ = ['MusicManager',
           'music',
           'SoundManager',
           'sound',]
__docformat__ = 'reStructuredText'

class MusicManager(resman.Submanager):
    """
    Game music manager.
    
    For sound effects, use `SoundManager`.
    """
    endEvent = 25
    
    def __init__(self,
                 should_play=True,
                 loop=True,
                 *args, **kw):
        super(MusicManager, self).__init__(*args, **kw)
        self.savedPlaylists = {}
        self.playlist = []
        self.loop = loop
        self.shouldPlay = should_play
        self.index = 0
        self.playing = False
        pygame.mixer.music.set_endevent(self.endEvent)
    
    # Playlist management
    
    def prepare(self, tag, playlist):
        """
        Prepares a playlist for playback later.
        
        .. Warning::
           `prepare` is retained for compatibility reasons.  You should be using
           `addPlaylist` to add new resources.
        """
        warnings.warn("prepare is deprecated; use addPlaylist.",
                      DeprecationWarning,
                      stacklevel=2)
        self.addPlaylist(tag, playlist)
    
    def addPlaylist(self, key, songs):
        """Adds a new playlist."""
        self.savedPlaylists[key] = list(songs)
    
    def getPlaylist(self, key):
        """Accesses the playlist."""
        return self.savedPlaylists[key]
    
    def removePlaylist(self, key):
        """Removes a saved playlist."""
        del self.savedPlaylists[key]
    
    # Playback control
    
    def startPlaylist(self, key):
        """
        Starts a playlist denoted by ``key``.
        
        You may pass an iterable object to `startPlaylist`, and the object will
        be used as a playlist.
        
        After calling `startPlaylist`, the playlist is *cued*.  You then need to
        call `play` to start playing music.
        """
        try:
            self.playlist = self.getPlaylist(key)
        except KeyError:
            try:
                self.playlist = list(key)
            except TypeError:
                raise TypeError("Playlist must be added or an iterable object")
        if self.playlist:
            self.index = 0
            self.loadSong()
    
    def play(self):
        """Starts the current song."""
        if self.shouldPlay and not self.playing:
            if pygame.mixer.music.get_busy():
                pygame.mixer.music.unpause()
            else:
                pygame.mixer.music.play()
            self.playing = True
    
    def pause(self):
        """
        Pauses the current song.
        
        You can resume using `play`.
        """
        pygame.mixer.music.pause()
        self.playing = False
    
    def stop(self):
        """Stops the current song and goes to the beginning."""
        pygame.mixer.music.stop()
        self.playing = False
    
    def load(self, key=None):
        """
        Manually loads a song.
        
        If no argument is given, the current song in the playlist is loaded.
        This method will not load anything unless the manager is configured to
        play.
        """
        if key is None:
            if self.playlist:
                key = self.playlist[self.index]
            else:
                raise ValueError("No song in the playlist")
        if self.shouldPlay:
            super(MusicManager, self).load(key)
    
    loadSong = load
    
    def previousSong(self):
        """Returns to the previous song in the playlist."""
        if self.playlist:
            self.index -= 1
            if self.index < 0:
                self.index = len(self.playlist) - 1
            self.stop()
            self.loadSong()
        
    def nextSong(self):
        """Advances to the next song in the playlist."""
        if self.playlist:
            playNextSong = self.playing
            self.index += 1
            if self.index >= len(self.playlist):
                self.index = 0
                if not self.loop:
                    playNextSong = False
            self.stop()
            self.loadSong()
            if playNextSong:
                self.play()
    
    # Volume control
    
    def getVolume(self):
        """
        Gets the current volume.
        
        You can also use the ``volume`` property.
        """
        return self.volume
    
    def setVolume(self, volume):
        """
        Changes the current volume.
        
        You can also use the ``volume`` property.
        """
        self.volume = volume
    
    def _getVolume(self):
        """Retrieves the mixer's current volume."""
        return pygame.mixer.music.get_volume()
    
    def _setVolume(self, volume):
        """Changes the mixer's current volume."""
        pygame.mixer.music.set_volume(volume)
    
    volume = property(_getVolume, _setVolume, doc="The mixer's volume")

music = MusicManager()

class SoundManager(resman.Submanager):
    """
    Sound effects manager.
    
    For music, use `MusicManager`.
    """
    resourceType = resman.SoundResource
    
    def __init__(self, should_play=True, volume=0.5, *args, **kw):
        super(SoundManager, self).__init__(*args, **kw)
        self.shouldPlay = should_play
        self.volume = volume
    
    def getSound(self, *args, **kw):
        """
        Retrieves a sound, using a cache if possible.

        .. Warning::
           `getSound` is deprecated, for favor of the Submanager API.  Use
           `load` instead.
        """
        warnings.warn("getSound is deprecated; use load.",
                      DeprecationWarning,
                      stacklevel=2)
        return self.load(*args, **kw)
    
    def play(self, tag, volume=None, cache=True):
        """
        Plays a sound and returns the sound object.
        
        This method will use a cached representation if possible.  Also, you can
        specify a non-default volume, if desired.  The cache flag specifies
        whether the sound will be cached, not whether it uses the cache.
        """
        if volume is None:
            volume = self.volume
        if self.shouldPlay:
            if cache:
                self.cache(tag)
            snd = self.load(tag)
            snd.set_volume(volume)
            snd.play()
            return snd
        else:
            return None

sound = SoundManager()



#!/usr/bin/env python
#
#   setup.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""setuptools script for pymage"""

# Run bootstrap setuptools installer
from ez_setup import use_setuptools
use_setuptools()

# Import modules
from setuptools import setup
import sys

__author__ = 'Ross Light'
__date__ = 'August 16, 2006'
__all__ = []

# Check for Python 2.4
if sys.version < '2.4.0':
    print >> sys.stderr, "pymage requires Python 2.4"
    sys.exit(1)

# Call setup function
setup(name="pymage",
      version='0.2.1',
      description="Pygame helper package",
      long_description="""\
pymage is a Python package that simplifies many aspects of Pygame programming
(e.g. resource loading).

Its features include:

* File-based configuration
* Auto-configuring joystick support
* Music playlists
* Vectors
* State machine
* High-level timing""",
      license='LGPL',
      author="Ross Light",
      author_email='rlight2@gmail.com',
      keywords="pygame SDL configuration resource",
      url='http://code.google.com/p/pymage/',
      download_url='http://code.google.com/p/pymage/downloads/list',
      classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: Games/Entertainment',],
      test_suite='tests',
      packages=['pymage'],
      install_requires=['pygame'],
      zip_safe=True,)



#!/usr/bin/env python
#
#   vector.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Manipulate mathematical vectors."""

from __future__ import division
import math

# Get Numeric Python, if we have it
numpy = None
for name in ('numpy', 'Numeric'):
    try:
        numpy = __import__(name, globals())
    except ImportError:
        pass
    else:
        break
del name

__author__ = 'Ross Light'
__date__ = 'March 3, 2006'
__all__ = ['Vector', 'i', 'j']
__docformat__ = 'reStructuredText'

scalarTypes = (int, float, long)

def _getComponents(args, kw):
    """Returns (x, y, z) triple for __init__ arguments."""
    specKw = ('x' in kw, 'y' in kw, 'z' in kw)
    x = kw.pop('x', 0.0)
    y = kw.pop('y', 0.0)
    z = kw.pop('z', 0.0)
    if len(kw) > 0:
        raise TypeError("Unrecognized keyword argument: %s" % list(kw)[0])
    if len(args) == 0:
        pass
    elif len(args) == 1:
        try:
            seq = list(args[0])
        except TypeError:
            # 1D vector
            if specKw[0]:
                raise TypeError("Multiple values for x")
            x = args[0]
        else:
            # Sequence unpack
            if True in specKw:
                raise TypeError("Keyword arguments not allowed on sequences")
            if len(seq) == 0:
                x = y = z = 0.0
            elif len(seq) == 1:
                (x,), y, z = seq, 0.0, 0.0
            elif len(seq) == 2:
                (x, y), z = seq, 0.0
            elif len(seq) == 3:
                (x, y, z) = seq
            else:
                raise TypeError("Sequence with more than 3 values")
    elif len(args) == 2:
        if specKw[0]:
            raise TypeError("Multiple values for x")
        if specKw[1]:
            raise TypeError("Multiple values for y")
        x, y = args
    elif len(args) == 3:
        if specKw[0]:
            raise TypeError("Multiple values for x")
        if specKw[1]:
            raise TypeError("Multiple values for y")
        if specKw[2]:
            raise TypeError("Multiple values for z")
        x, y, z = args
    else:
        raise TypeError("Unrecognized vector creation parameters")
    return (float(x), float(y), float(z))

class Vector(object):
    """A three-dimensional vector."""
    
    def __new__(cls, *args, **kw):
        """
        ``Vector(x, y, z)`` -> `Vector`
        ``Vector((x, y, z))`` -> `Vector`
        
        You don't have to specify all three coordinates, only those necessary.
        """
        if cls is Vector:
            if numpy is None:
                return PythonVector.__new__(PythonVector, *args, **kw)
            else:
                return NumericVector.__new__(NumericVector, *args, **kw)
        else:
            return super(Vector, cls).__new__(cls, *args, **kw)
    
    @classmethod
    def findVector(cls, angle, magnitude):
        """
        Finds the two-dimensional vector with the given angle and magnitude.
        
        The angle is in degrees counterclockwise from the positive x-axis.
        Round-off may experience round-off error.
        """
        # Put angle in range of [0, 360)
        while angle < 0:
            angle += 360
        while angle >= 360:
            angle -= 360
        # Find angle in radians
        radAngle = math.radians(angle)
        # Catch special cases
        if angle == 90 or angle == 270:
            cos = 0
        else:
            cos = math.cos(radAngle)
        if angle == 0 or angle == 180:
            sin = 0
        else:
            sin = math.sin(radAngle)
        # Create vector
        return cls(cos * magnitude, sin * magnitude)
    
    @classmethod
    def twoPointVector(cls, point1, point2):
        """
        Finds the vector with the given points.
        
        The points can be one-, two-, or three-dimensional.
        """
        return cls(point2) - cls(point1)
    
    # String conversion
    
    def __repr__(self):
        cls = type(self)
        name = '%s.%s' % (cls.__module__, cls.__name__)
        if not self:
            return '%s()' % (name)
        elif self.z == 0:
            return '%s(%g, %g)' % (name, self.x, self.y)
        else:
            return '%s(%g, %g, %g)' % (name, self.x, self.y, self.z)
    
    def __str__(self):
        if self.z == 0:
            return '<%g, %g>' % (self.x, self.y)
        else:
            return '<%g, %g, %g>' % (self.x, self.y, self.z)
    
    # Operations
    
    def __neg__(self):
        return type(self)(-self.x, -self.y, -self.z)
    
    def __pos__(self):
        return type(self)(+self.x, +self.y, +self.z)
    
    def __add__(self, other):
        if isinstance(other, Vector):
            return type(self)(self.x + other.x,
                              self.y + other.y,
                              self.z + other.z)
        else:
            return NotImplemented
    
    def __sub__(self, other):
        if isinstance(other, Vector):
            return type(self)(self.x - other.x,
                              self.y - other.y,
                              self.z - other.z)
        else:
            return NotImplemented
        
    def __mul__(self, other):
        """
        ``v * x <==> v.__mul__(x)``
        
        x can be a scalar (scalar multiplication) or a `Vector` (dot product).
        """
        if isinstance(other, scalarTypes):      # Scalar
            other = float(other)
            return type(self)(self.x * other,
                              self.y * other,
                              self.z * other)
        elif isinstance(other, Vector):         # Dot Product
            return (self.x * other.x +
                    self.y * other.y +
                    self.z * other.z)
        else:
            return NotImplemented
    
    def __rmul__(self, other):
        return self.__mul__(other)
    
    def __div__(self, other):
        # Do true division by default, because that's what one would expect,
        # since our instance variables are floats.
        return self.__truediv__(other)
        
    def __truediv__(self, other):
        if isinstance(other, scalarTypes):      # Scalar
            other = float(other)
            return type(self)(self.x / other,
                              self.y / other,
                              self.z / other)
        else:
            return NotImplemented
        
    def __floordiv__(self, other):
        if isinstance(other, scalarTypes):      # Scalar
            return type(self)(self.x // other,
                              self.y // other,
                              self.z // other)
        else:
            return NotImplemented
    
    def proj(self, other):
        """Mathematically: projv u for ``v.proj(u)``"""
        return (other * self / self.magnitude ** 2) * self
    
    def angleBetween(self, other):
        """Obtains the angle between the two vectors in degrees."""
        radianAngle = math.acos((self * other) /
                                (self.magnitude * other.magnitude))
        return math.degrees(radianAngle)
    
    def unitVector(self):
        """Creates a vector in the same direction, but of magnitude 1."""
        return self / self.magnitude
    
    def _calcMagnitude(self):
        """Calculate magnitude."""
        if numpy is None:
            sqrt = math.sqrt
        else:
            sqrt = numpy.sqrt
        mag = sqrt(self.x ** 2 + self.y ** 2 + self.z ** 2)
        return float(mag)
    
    def _calcAngle(self):
        """Calculate angle in degrees, ignoring z-axis."""
        if numpy is None:
            if self.x == 0:
                if self.y > 0:
                    deg = 90.0
                elif self.y < 0:
                    deg = -90.0
                else:
                    deg = 0.0
            else:
                deg = math.degrees(math.atan(self.y / self.x))
                while deg < 0:
                    deg += 360
                while deg > 360:
                    deg -= 360
        else:
            deg = math.degrees(numpy.angle(numpy.complex(self.x, self.y)))
        return float(deg)
    
    # List conversion
    
    def __iter__(self):
        """
        ``iter(v) <==> v.__iter__()``
        
        In sequence, this returns the components of the vector, and leaving z
        out if it is 0.  This is usually what you want, but you can force 2D or
        3D by using `iter2D` and `iter3D`, respectively.
        """
        yield self.x
        yield self.y
        if self.z != 0:
            yield self.z
    
    def iter2D(self):
        """Returns an iterator that accesses only the x and y components."""
        yield self.x
        yield self.y
    
    def iter3D(self):
        """Returns an iterator that accesses the x, y, and z components."""
        yield self.x
        yield self.y
        yield self.z
    
    def list2D(self):
        """Returns a list that contains only the x and y components."""
        return [self.x, self.y]
    
    def list3D(self):
        """Returns a list that contains the x, y, and z components."""
        return [self.x, self.y, self.z]
    
    # Comparison
    
    def __eq__(self, other):
        if isinstance(other, Vector):
            return (self.x == other.x and
                    self.y == other.y and
                    self.z == other.z)
        else:
            return NotImplemented
    
    def __ne__(self, other):
        if isinstance(other, Vector):
            return (self.x != other.x or
                    self.y != other.y or
                    self.z != other.z)
        else:
            return NotImplemented
    
    def __nonzero__(self):
        return bool(self.x or self.y or self.z)
    
    def __hash__(self):
        return int(self.x) ^ int(self.y) ^ int(self.z)
    
    # Component access
    
    def __setattr__(self, attr, value):
        raise AttributeError("Vector is an immutable object")
    
    def __delattr__(self, attr):
        raise AttributeError("Vector is an immutable object")
    
    def _setattr(self, attr, value):
        """
        The real implementation of __setattr__.
        
        But don't tell anyone!  It's a *secret*.
        """
        super(Vector, self).__setattr__(attr, value)
    
    @property
    def x(self):
        raise NotImplementedError()
    
    @property
    def y(self):
        raise NotImplementedError()
    
    @property
    def z(self):
        raise NotImplementedError()
    
    @property
    def magnitude(self):
        """The length of the vector."""
        try:
            return self._magnitude
        except AttributeError:
            mag = self._calcMagnitude()
            self._setattr('_magnitude', mag)
            return mag
    
    @property
    def angle(self):
        """The angle in degrees from the positive x-axis."""
        try:
            return self._angle
        except AttributeError:
            deg = self._calcAngle()
            self._setattr('_angle', deg)
            return deg

class PythonVector(Vector):
    """Vector implemented in pure Python."""
    
    __slots__ = ['x', 'y', 'z', '_magnitude', '_angle']
    
    def __init__(self, *args, **kw):
        x, y, z = _getComponents(args, kw)
        self._setattr('x', x)
        self._setattr('y', y)
        self._setattr('z', z)

class NumericVector(Vector):
    __slots__ = ['_array', '_magnitude', '_angle']
    
    def __init__(self, *args, **kw):
        x, y, z = _getComponents(args, kw)
        self._setattr('_array', numpy.array([x, y, z]))
    
    # Operations
    
    def __add__(self, other):
        if isinstance(other, NumericVector):
            return type(self)(self._array + other._array)
        else:
            return super(NumericVector, self).__add__(other)
    
    def __sub__(self, other):
        if isinstance(other, NumericVector):
            return type(self)(self._array - other._array)
        else:
            return super(NumericVector, self).__sub__(other)
        
    def __mul__(self, other):
        if isinstance(other, scalarTypes):      # Scalar
            return type(self)(self._array * float(other))
        elif isinstance(other, NumericVector):  # Dot Product
            return float(numpy.dot(self._array, other._array))
        else:
            return super(NumericVector, self).__mul__(other)
    
    def __rmul__(self, other):
        if isinstance(other, scalarTypes):
            return self.__mul__(other)
        else:
            return NotImplemented
    
    def __div__(self, other):
        # Do true division by default, because that's what one would expect,
        # since our instance variables are floats.
        return self.__truediv__(other)
        
    def __truediv__(self, other):
        if isinstance(other, scalarTypes):      # Scalar
            return type(self)(self._array / float(other))
        else:
            return super(NumericVector, self).__truediv__(other)
        
    def __floordiv__(self, other):
        if isinstance(other, scalarTypes):      # Scalar
            return type(self)(self._array / float(other))
        else:
            return super(NumericVector, self).__floordiv__(other)
    
    # Component access
    
    @property
    def x(self):
        return float(self._array[0])
    
    @property
    def y(self):
        return float(self._array[1])
    
    @property
    def z(self):
        return float(self._array[2])

# Special vectors
i = Vector(1, 0)
j = Vector(0, 1)



#!/usr/bin/env python
#
#   resmantest.py
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

import os
import unittest

import pygame
from pymage.resman import *

__author__ = 'Ross Light'
__date__ = 'February 28, 2007'
__all__ = ['ResourceManagerTestCase', 'test_suite',]

resourceDirectory = os.path.join(os.path.dirname(__file__), 'data')

class ResourceManagerTestCase(unittest.TestCase):
    resources = {'TestImage': ('test_img.png', ImageResource, [False]),
                 'HelloWorld': ('hello.ogg', SoundResource),}
    groupName = 'group'
    
    def setUp(self):
        # Initialize pygame mixer system
        pygame.mixer.init()
        # Create resource manager
        self.resman = ResourceManager()
        # Add resources to manager
        for key, info in self.resources.iteritems():
            name, resType = info[:2]
            if len(info) == 2:
                args, kw = tuple(), dict()
            elif len(info) == 3:
                args, kw = info[2], dict()
            elif len(info) == 4:
                args, kw = info[2:4]
            else:
                raise RuntimeError("Bad resource description: %r" % info)
            resPath = os.path.join(resourceDirectory, name)
            self.resman.addResource(key, resType(resPath, *args, **kw))
        # Add group with all resources
        self.resman.addCacheGroup(self.groupName, self.resources.keys())
    
    def testIndividualCache(self):
        """Individual resource cache/uncache test"""
        for key in self.resources:
            self.resman.cacheResource(key)
            self.assert_(self.resman.getResource(key).cache is not None,
                         "%r did not create cache" % (key,))
            self.resman.uncacheResource(key)
            self.assert_(self.resman.getResource(key).cache is None,
                         "%r did not destroy cache" % (key,))
    
    def testGroupCache(self):
        """Group resource cache/uncache test"""
        self.resman.cacheGroup(self.groupName)
        for key in self.resources:
            self.assert_(self.resman.getResource(key).cache is not None,
                         "%r did not create cache" % (key,))
        self.resman.uncacheGroup(self.groupName)
        for key in self.resources:
            self.assert_(self.resman.getResource(key).cache is None,
                         "%r did not destroy cache" % (key,))
    
    def testResourcePaths(self):
        """Resource path correctness test"""
        for key, info in self.resources.iteritems():
            resourcePath = self.resman.getResource(key).path
            infoPath = os.path.join(resourceDirectory, info[0])
            self.assertEqual(resourcePath, infoPath,
                             "%r path %r does not match %r)" % (key,
                                                                resourcePath,
                                                                infoPath))
    
    def testRemoval(self):
        """Resource removal test"""
        for key in self.resources:
            self.resman.removeResource(key)
            self.assertRaises(KeyError, self.resman.getResource, key)

test_suite = unittest.makeSuite(ResourceManagerTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#!/usr/bin/env python
#
#   config.py
#   pymage
#
#   Copyright (C) 2006-2007 Ross Light
#
#   This file is part of pymage.
#
#   pymage is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   pymage is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with pymage; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""
Provide configuration for your game.

The typical use of this module is to use the `setup` function and give set up
parameters.  From there, the module sets up the correct parts of the pymage
library.

This module is not strictly necessary, but pymage's reasoning is as follows:

Resources for the game should be loaded on an location-independent basis.
This provides abstraction and the ability for arbitrary file structure.
However, these files eventually need to be located, so the developer should
create a file (separate from the main code) that locates everything.  From
there, there may be a legitimate reason for the user to want to customize the
file locations (perhaps he/she wants the game resources to be on a separate hard
drive).  Therefore, a separate file is used to override the behavior of any
given installation of the game.  But since the user never touches the
developer's default game setup, any corrupt preferences can be corrected by
deleting the user configuration files.

Here is a sample gamesite.xml file::

    <?xml version="1.0"?>
    
    <!-- name is not necessary, but reserved for future use. -->
    <game-site name="Sample Game">
        <!-- Prepare an image in the ImageManager.
             The id attribute is used as the tag.
             The path element is required to find the file.
             The section and option elements specify how the path can be
             overriden in configuration files. -->
        <image id="SampleImage">
            <section>sprites</section>
            <option>sample</option>
            <path>img/sample.png</path>
        </image>
        
        <!-- Prepare a sound in the SoundManager.
             The id attribute is used as the tag.
             The path element is required to find the file.
             The section and option elements specify how the path can be
             overriden in configuration files. -->
        <sound id="SampleSound">
            <section>sounds</section>
            <option>sample</option>
            <path>sfx/sample.wav</path>
        </sound>
        
        <!-- Prepare a playlist in the MusicManager.
             The id attribute is used as the tag.
             The path element(s) are required to find the music file(s).
             The section and option elements specify how the playlist can be
             overriden in configuration files. -->
        <music id="song1">
            <path>music/song1.wav</path>
        </music>
        <playlist id="SamplePlaylist">
            <section>playlists</section>
            <option>sample</option>
            <music ref="song1"/>
            <path>music/song2.wav</path> <!-- Old way, still works -->
        </playlist>
        
        <!-- Prepare a cache group. -->
        <group id="SampleGroup">
            <section>groups</section>
            <option>sample</option>
            <image ref="SampleImage"/>
            <sound ref="SampleSound"/>
        </group>
        
        <!-- Specify additional configuration files. -->
        <config-file>userconfig.ini</config-file>
    </game-site>
"""

from ConfigParser import ConfigParser
import os
from textwrap import dedent
import warnings
from xml.dom import minidom

from pymage import resman
from pymage import sound

__author__ = 'Ross Light'
__date__ = 'August 10, 2006'
__all__ = ['GameSiteWarning',
           'load',
           'save',
           'getOption',
           'setOption',
           'registerType',
           'unregisterType',
           'setup',]
__docformat__ = 'reStructuredText'

# Globals
_gsPrims = {'resource': resman.Resource,
            'image': resman.ImageResource,
            'sound': resman.SoundResource,
            'music': resman.MusicResource,}

class GameSiteWarning(UserWarning):
    """Warning emitted when odd game site constructs are used."""
    pass

## CONFIG PARSER ##

class CaseConfigParser(ConfigParser):
    """A ``ConfigParser`` that is case-sensitive."""
    def optionxform(self, optstr):
        """Return the string in the same case."""
        return optstr

def load(*args, **kw):
    """
    Load a series of configuration files.
    
    This function returns a dictionary of values.  Any optional keyword values
    are used as variables.  The special keyword ``convert`` specifies whether
    the function should interpret the values as what they seem to be
    (e.g. ``float``, ``int``).  The default is ``True``.
    """
    # Retrieve convert keyword
    convertValues = kw.get('convert', True)
    try:
        del kw['convert']
    except KeyError:
        pass
    # Parse the files
    parser = CaseConfigParser(kw)
    for configFile in args:
        close = False
        if isinstance(configFile, basestring):
            # Open strings as paths
            path = os.path.normpath(os.path.expanduser(configFile))
            if os.path.exists(path):
                configFile = open(path)
            else:
                continue
            close = True
        parser.readfp(configFile)
        if close:
            configFile.close()
    # Assemble dictionary
    configDict = {}
    for section in parser.sections():
        sectionDict = {}
        for option in parser.options(section):
            value = parser.get(section, option)
            if convertValues:   # Interpret values
                value = _getValue(value)
            sectionDict[option] = value
        configDict[section] = sectionDict
    return configDict

def save(config, configFile):
    """Saves a configuration dictionary to a file."""
    close = False
    parser = CaseConfigParser()
    for section, values in config.iteritems():
        for option, value in values.iteritems():
            parser.set(section, option, value)
    if isinstance(configFile, basestring):
        path = os.path.normpath(os.path.expanduser(configFile))
        configFile = open(path, 'w')
        close = True
    parser.write(configFile)
    if close:
        configFile.close()

def _getValue(value_string):
    """Retrieves a value from a ``ConfigParser`` string."""
    boolLiterals = {'false': False,
                    'no': False,
                    'off': False,
                    'true': True,
                    'yes': True,
                    'on': True,}
    if value_string.isdigit():
        # Integer
        return int(value_string)
    elif value_string.lower() in boolLiterals:
        # Boolean
        return boolLiterals[value_string.lower()]
    elif _isFloat(value_string):
        # Float
        return float(value_string)
    else:
        # String
        return str(value_string)

def _isFloat(value_string):
    """
    Returns whether the string is a ``float``.
    
    The format for a float is::
    
        int[.[fraction]]
    """
    if value_string.isdigit():
        # int
        return True
    elif value_string.count('.') == 1:
        # has a decimal point
        value_string = value_string.replace('.', '')
        if value_string.isdigit():
            # int[.[fraction]]
            return True
        else:
            return False
    else:
        return False

def getOption(config, section, option, default=None):
    """Retrieves an option from the configuration dictionary."""
    try:
        section = config[section]
    except KeyError:
        return default
    else:
        return section.get(option, default)

def setOption(config, section, option, value):
    """Changes an option in the configuration dictionary."""
    section = config.setdefault(section, {})
    section[option] = value

## GAME SITE ##

def registerType(tag, factory):
    """
    Register a custom game site resource type.
    
    The ``tag`` argument is the name of the XML element, and ``factory`` is a
    callable that takes one argument: a path to the resource.  The ``factory``
    is typically the constructor of a `resman.Resource` subclass.
    """
    _gsPrims[tag] = factory

def unregisterType(tag):
    """Unregister a custom game site resource type."""
    del _gsPrims[tag]

def setup(site='gamesite.xml', *config_files, **kw):
    """
    Sets up a game from the specified parameters and returns the configuration
    dictionary.
    
    The first argument is the game site file.  The game site file contains most
    of the setup information.  It specifies the resource IDs and where to find
    the configuration files.  It should only be modified by developers as the
    game is created.
    
    The additional arguments are program default configuration files.  These
    are parsed before any inside the game site file (therefore giving the site
    configuration files higher precedence).
    
    Additional keyword arguments can be specified. ``configSound`` specifies
    whether the sound manager volume is configured, and ``configMusic``
    specifies whether the music manager volume is configured.
    """
    # Get keyword arguments
    configSound = kw.pop('configSound', True)
    configMusic = kw.pop('configMusic', True)
    if kw:
        raise TypeError("Invalid keyword argument")
    # Parse game site file
    doc = minidom.parse(site)
    config = _getSiteConfig(doc, config_files)
    # Load configuration
    if configSound:
        _processSoundOptions(config)
    if configMusic:
        _processMusicOptions(config)
    # Process resources
    _processGameSite(doc, config)
    # Return configuration dictionary
    return config

def _getSiteConfig(doc, config_files):
    """
    Find the full list of configuration files.
    
    The configuration files passed in are put first in the list, so the ones
    specified in the game site file take precedence.
    """
    siteConfigs = []
    for child in doc.documentElement.childNodes:
        if (child.nodeType == minidom.Node.ELEMENT_NODE and
            child.tagName == 'config-file'):
            siteConfigs.append(_getText(child))
    config_files = list(config_files) + list(siteConfigs)
    return load(*config_files)

def _processSoundOptions(config):
    """Configure the sound manager with the default configuration keys."""
    sound.sound.shouldPlay = bool(getOption(config, 'sound', 'play', True))
    sound.sound.volume = float(getOption(config, 'sound', 'volume', 1.0))

def _processMusicOptions(config):
    """Configure the music manager with the default configuration keys."""
    sound.music.shouldPlay = bool(getOption(config, 'music', 'play', True))
    sound.music.volume = bool(getOption(config, 'music', 'volume', 0.5))
    sound.music.loop = bool(getOption(config, 'music', 'loop', True))

def _processGameSite(doc, config):
    """Run through game site file and pull out resources."""
    handlers = {'playlist': _handlePlaylist,
                'group': _handleGroup,}
    handlers.update(dict.fromkeys(_gsPrims, _handlePrimitive))
    for child in doc.documentElement.childNodes:
        if (child.nodeType == minidom.Node.ELEMENT_NODE and
            child.tagName in handlers):
            # Call handler
            handler = handlers[child.tagName]
            handler(child, config)

def _handlePrimitive(elem, config):
    """
    Handle a basic resource (i.e. images, sound effects, and custom resources).
    """
    attr = _attributes(elem, include_ns=False, ascii=True)
    pathChild = _childNamed(elem, 'path')
    if pathChild is None:
        warnings.warn("Primitive without a path", GameSiteWarning)
        return
    # Get ID
    try:
        key = attr.pop('id')
    except KeyError:
        pathChild = _childNamed(elem, 'path')
        if pathChild:
            key = _getText(pathChild)
        else:
            warnings.warn("Primitive without a key", GameSiteWarning)
            return
    # Get resource information
    resType = _gsPrims[elem.tagName]
    section = _getText(_childNamed(elem, 'section'))
    option = _getText(_childNamed(elem, 'option'))
    path = _getText(pathChild)
    # Create resource
    if section is not None and option is not None:
        path = getOption(config, section, option, path)
    resman.resman.addResource(key, resType(path, **attr))
    # Return key
    return key

def _handlePlaylist(elem, config):
    """Handle a playlist element."""
    key = elem.getAttribute('id')
    section = _getText(_childNamed(elem, 'section'))
    option = _getText(_childNamed(elem, 'option'))
    playlistKeys = []
    # Get playlist keys
    for sub in elem.childNodes:
        if sub.nodeType == minidom.Node.ELEMENT_NODE:
            if sub.tagName == 'path':
                # Old-school path approach
                warnings.warn("%s using old path-based playlist" % (key),
                              GameSiteWarning)
                if sub.hasAttribute('id'):
                    musicKey = sub.getAttribute('id')
                    musicPath = _getText(sub)
                else:
                    musicKey = musicPath = _getText(sub)
                resman.addResource(musicKey, resman.MusicResource(musicPath))
                playlistKeys.append(musicKey)
            elif sub.tagName == 'music':
                # New-school music reference/declaration approach
                if sub.hasAttribute('ref'):
                    musicKey = sub.getAttribute('ref')
                else:
                    musicKey = _handlePrimitive(sub, config)
                playlistKeys.append(musicKey)
    # Create playlist
    if section is not None and option is not None:
        configKeys = getOption(config, section, option)
        if configKeys is not None:
            playlistKeys = configKeys.split(',')
    sound.music.addPlaylist(key, playlistKeys)
    # Return key
    return key

def _handleGroup(elem, config):
    """Handle a group element (a cache group)."""
    key = elem.getAttribute('id')
    section = _getText(_childNamed(elem, 'section'))
    option = _getText(_childNamed(elem, 'option'))
    groupKeys = set()
    # Get group keys
    for sub in elem.childNodes:
        if (sub.nodeType == minidom.Node.ELEMENT_NODE and
            sub.tagName in _gsPrims):
            if sub.hasAttribute('ref'):
                resourceKey = sub.getAttribute('ref')
            else:
                resourceKey = _handlePrimitive(sub, config)
            groupKeys.add(resourceKey)
    # Create group
    if section is not None and option is not None:
        configKeys = getOption(config, section, option)
        if configKeys is not None:
            groupKeys = configKeys.split(',')
    resman.resman.addCacheGroup(key, groupKeys)
    # Return key
    return key

def _getText(elem, post=True):
    """
    Retrieve text from a DOM node, stripping indents, if asked.
    
    This function does honor the ``xml:space`` attribute, and if
    ``xml:space="preserve"`` is specified, it takes precendence over the
    ``post`` argument.
    """
    xmlNS = 'http://www.w3.org/XML/1998/namespace'
    if elem is None:
        return None
    text = ''
    for child in elem.childNodes:
        if child.nodeType == minidom.Node.TEXT_NODE:
            text += child.wholeText
    preserve = (elem.hasAttributeNS(xmlNS, 'space') and
                elem.getAttributeNS(xmlNS, 'space') == 'preserve')
    if post and not preserve:
        text = dedent(text)
        if text.startswith('\n'):
            text = text[1:]
            if text.endswith('\n'):
                text = text[:-1]
    return text

def _childNamed(elem, name):
    """Returns the first child with the given name, or ``None`` if not found."""
    for child in elem.childNodes:
        if (child.nodeType == minidom.Node.ELEMENT_NODE and
            child.tagName == name):
            return child
    else:
        return None

def _attributes(elem, include_ns=True, ascii=False):
    """
    Retrieves the attributes of a DOM node as a dictionary.
    
    If ``include_ns`` is ``True``, then attributes with a namespace will be
    discarded.  If ``ascii`` is ``True``, then the attribute names are converted
    to ASCII, but if they are unable to do so, the attribute is discarded.
    """
    nodemap = elem.attributes
    attrDict = {}
    for index in xrange(nodemap.length):
        attr = nodemap.item(index)
        if not include_ns and attr.prefix:
            continue
        name = attr.localName
        if ascii:
            try:
                name = str(name)
            except UnicodeError:
                continue
        attrDict[name] = attr.value
    return attrDict
